// 320x240LCD
// JPEG表示

#include <plib.h>
#include "LCDdriver.h"
#include "FSIO.h"
#include "ImageDecoder.h"

// DEVCFG3
// USERID = No Setting
#pragma config PMDL1WAY = OFF           // Peripheral Module Disable Configuration (Allow multiple reconfigurations)
#pragma config IOL1WAY = OFF            // Peripheral Pin Select Configuration (Allow multiple reconfigurations)
#pragma config FUSBIDIO = OFF           // USB USID Selection (Controlled by Port Function)
#pragma config FVBUSONIO = OFF          // USB VBUS ON Selection (Controlled by Port Function)

// DEVCFG2
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider (2x Divider)
#pragma config FPLLMUL = MUL_24         // PLL Multiplier (24x Multiplier)
#pragma config FPLLODIV = DIV_2         // System PLL Output Clock Divider (PLL Divide by 2)

// DEVCFG1
#pragma config FNOSC = FRCPLL           // Oscillator Selection Bits (Fast RC Osc with PLL)
#pragma config FSOSCEN = OFF            // Secondary Oscillator Enable (Disabled)
#pragma config IESO = OFF               // Internal/External Switch Over (Disabled)
#pragma config POSCMOD = OFF            // Primary Oscillator Configuration (Primary osc disabled)
#pragma config OSCIOFNC = OFF           // CLKO Output Signal Active on the OSCO Pin (Disabled)
#pragma config FPBDIV = DIV_1           // Peripheral Clock Divisor (Pb_Clk is Sys_Clk/1)
#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor Selection (Clock Switch Disable, FSCM Disabled)
#pragma config WDTPS = PS1048576        // Watchdog Timer Postscaler (1:1048576)
#pragma config WINDIS = OFF             // Watchdog Timer Window Enable (Watchdog Timer is in Non-Window Mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (WDT Disabled (SWDTEN Bit Controls))
//#pragma config FWDTWINSZ = WINSZ_25     // Watchdog Timer Window Size (Window Size is 25%)

// DEVCFG0
#pragma config JTAGEN = OFF             // JTAG Enable (JTAG Disabled)
#pragma config ICESEL = ICS_PGx1        // ICE/ICD Comm Channel Select (Communicate on PGEC1/PGED1)
#pragma config PWP = OFF                // Program Flash Write Protect (Disable)
#pragma config BWP = OFF                // Boot Flash Write Protect bit (Protection Disabled)
#pragma config CP = OFF                 // Code Protect (Protection Disabled)

// 入力ボタンのポート、ビット定義
#define KEYPORT PORTB
#define KEYDOWN  0x0004
#define KEYLEFT  0x0008
#define KEYRIGHT 0x0010
#define KEYUP    0x0020
#define KEYSTART 0x0080
#define KEYFIRE  0x0100
#define KEYMASK (KEYSTART | KEYFIRE | KEYUP | KEYDOWN | KEYLEFT | KEYRIGHT)
#define KEY_DS LATASET=0x0008
#define KEY_EN LATACLR=0x0008

volatile unsigned short drawcount;
/**********************
*  Timer2 割り込み処理関数
***********************/
void __ISR(8, ipl4) T2Handler(void)
{
	drawcount++;
	IFS0bits.T2IF = 0;			// T2割り込みフラグクリア
}

void photo(void){
	SearchRec sr;
	IMG_FILE *pImageFile;
	volatile unsigned int pb;
	//	IMG_vInitialize();
	ImageDecoderInit();
	while(1){
		FindFirst("*.jpg",ATTR_MASK,&sr);
		do{
			pImageFile=IMG_FOPEN(sr.filename,READ); // 320*240 image
			LCD_Clear(0);//画面消去
			ImageDecode(pImageFile, IMG_JPEG, 0, 0, 320, 240, IMG_DOWN_SCALE | IMG_ALIGN_CENTER, NULL, NULL);
			IMG_FCLOSE(pImageFile);
			drawcount=0;
			while(drawcount<5*60) asm("wait");
		}
		while(!FindNext(&sr));
	}

}

void main(void){

	//ポートの初期設定
	TRISA = 0x0000; // PORTA全て出力
	ANSELA = 0x0000; // 全てデジタル
	TRISB = 0x0000; // PORTB全て出力
	ANSELB = 0x0000; // 全てデジタル
	KEY_DS; // キー読み込み無効

	// 周辺機能ピン割り当て
	SDI2R=2; //RPA4:SDI2
	RPA1R=4; //RPA1:SDO2

	INTEnableSystemMultiVectoredInt();
	IPC2bits.T2IP = 4;			// 割り込みレベル4
	IFS0bits.T2IF = 0;
	IEC0bits.T2IE = 1;			// タイマ2割り込み有効化
	PR2=3125;				// 60分の1秒
	T2CON = 0x8070;				// プリスケーラ1:256、タイマ2開始

	LCD_set_Horizontal();
	LCD_Clear(0);
	if(FSInit()==FALSE){ //ファイルシステム初期化
		//エラーの場合停止
		while(1) asm("wait");
	}
	photo();
	while(1) asm("wait");
}
